-- Hi Emacs, this is -*- mode: vhdl; -*-
----------------------------------------------------------------------------------------------------
--
-- Trigger Unit for DSO -cm 5/2009
-- triggert wenn adc_wert > trig_level und
library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_unsigned.all;
  use ieee.numeric_std.all;


entity trigger_count is
  port (
    trig_reset : in  std_logic;              -- asyncronous reset
    adc_clk   : in  std_logic;
    trig_ena   : in  std_logic;
	adc_wert: in std_logic_vector(7 downto 0);
	trig_level: in std_logic_vector(7 downto 0);
	hyst_level: in std_logic_vector(7 downto 0);
    write_clk   : out  std_logic;
    done   : out  std_logic;
    trig_ok   : out  std_logic;
	trig_addr: out std_logic_vector(10 downto 0);
	addr: out std_logic_vector(10 downto 0)
    );
end trigger_count;

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
architecture arch of trigger_count is
	type stateT is (pretrigger,waiting,triggered,stopped);
	signal state : stateT := waiting;
	signal addr_c : std_logic_vector(10 downto 0);
	signal samples_since_triggered : std_logic_vector(10 downto 0);
	signal last_adc_wert : std_logic_vector(7 downto 0);
	signal write_clk_ena : std_logic;
	signal pos_edge : std_logic;
	signal neg_edge : std_logic;
begin

addr <= addr_c;
write_clk <= adc_clk and write_clk_ena;

process(trig_reset, adc_clk)
begin
	if trig_reset = '1' then
		addr_c <= (others =>'0');
		trig_addr <= (others =>'0');
		samples_since_triggered <= (others =>'0');
		state<=pretrigger;
		write_clk_ena<='1';		
		done <= '0';
		trig_ok <= '0';
		last_adc_wert<=x"FF";	-- fr ansteigende Flanke
	else
		if rising_edge(adc_clk) then			
			case state is
			when pretrigger =>
				pos_edge <= '0';
				neg_edge <= '0';
				if (addr_c=x"3FF") then	-- immer 1024 Werte vor Trigger aufzeichnen
					state<=waiting;
				end if;
			when waiting =>
				if last_adc_wert<hyst_level then
					pos_edge <= '1';
				end if;
				if last_adc_wert>hyst_level then
					neg_edge <= '1';
				end if;
				-- auf steigende Flanke triggern
				if (adc_wert > trig_level) and (pos_edge='1') and (trig_ena='1') then
					state<=triggered;				
					trig_addr<=addr_c;
					trig_ok <= '1';
				end if;
				last_adc_wert<=adc_wert;
			when triggered =>
				if (samples_since_triggered=x"3FF") then	-- 1024 Werte aufgezeichnet
					state<=stopped;
					write_clk_ena<='0';		
				else
					samples_since_triggered <= samples_since_triggered + 1;
				end if;
			when stopped =>
				done <= '1';
			end case;
			addr_c <= addr_c + 1;	-- Ringzhler
		end if;
	end if;
end process;

end arch;
