-- Oszi wave RAM 2kx8 by -cm
-- Definition of a single port ROM for KCPSM3 program defined by pico_rom.psm
--
-- Standard IEEE libraries
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
-- The Unisim Library is used to define Xilinx primitives. It is also used during
-- simulation. The source can be viewed at %XILINX%\vhdl\src\unisims\unisim_VCOMP.vhd
--  
library unisim;
use unisim.vcomponents.all;
--
--
entity wave_ram is
    Port (read_addr : in std_logic_vector(10 downto 0);
			write_addr : in std_logic_vector(10 downto 0);
			trig_addr : in std_logic_vector(10 downto 0);
			read_data : out std_logic_vector(7 downto 0);
			read_clk : in std_logic;
			write_data: in std_logic_vector(7 downto 0);
			write_clk : in std_logic
			); 
    end wave_ram;

architecture behave of wave_ram is
--  <-----Cut code below this line and paste into the architecture body---->

   -- RAMB16_S18_S18: Virtex-II/II-Pro, Spartan-3/3E 1k x 16 + 2 Parity bits Dual-Port RAM
   -- Xilinx HDL Language Template, version 10.1.2
signal ram_addr: std_logic_vector(10 downto 0);

begin
--
  -- RAMB16_S9_S9: Virtex-II/II-Pro, Spartan-3/3E 2k x 8 + 1 Parity bit Dual-Port RAM
   -- Xilinx HDL Language Template, version 10.1.3

   RAMB16_S9_S9_inst : RAMB16_S9_S9
   generic map (
      INIT_A => X"000", --  Value of output RAM registers on Port A at startup
      INIT_B => X"000", --  Value of output RAM registers on Port B at startup
      SRVAL_A => X"000", --  Port A ouput value upon SSR assertion
      SRVAL_B => X"000", --  Port B ouput value upon SSR assertion
      WRITE_MODE_A => "WRITE_FIRST", --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      WRITE_MODE_B => "WRITE_FIRST", --  WRITE_FIRST, READ_FIRST or NO_CHANGE
      SIM_COLLISION_CHECK => "ALL") -- "NONE", "WARNING", "GENERATE_X_ONLY", "ALL" 
      -- The following INIT_xx declarations specify the initial contents of the RAM
      -- Address 0 to 511
      -- keine INITs, RAM vorerst leer
   port map (
      DOA => read_data(7 downto 0),      -- Port A 16-bit Data Output
      DOB => open,      -- Port B 8-bit Data Output
      DOPA => open,     -- Port A 1-bit Parity Output
      DOPB => open,     -- Port B 1-bit Parity Output
      ADDRA => ram_addr,      -- Port A 11-bit Address Input
      ADDRB => write_addr,    -- Port B 11-bit Address Input
      CLKA => read_clk,       -- Port A Clock
      CLKB => write_clk,      -- Port B Clock
      DIA => "00000000",      -- Port A 8-bit Data Input
      DIB => write_data,      -- Port B 16-bit Data Input
      DIPA => "0",     -- Port A 1-bit parity Input
      DIPB => "0",     -- Port-B 1-bit parity Input
      ENA => '1',      -- Port A RAM Enable Input
      ENB => '1',      -- PortB RAM Enable Input
      SSRA => '0',     -- Port A Synchronous Set/Reset Input
      SSRB => '0',     -- Port B Synchronous Set/Reset Input
      WEA => '0',      -- Port A Write Enable Input
      WEB => '1'       -- Port B Write Enable Input
   );

-- Triggeraddresse zum Referenz-Nullpunkt machen
ram_addr <= read_addr + trig_addr;
 
end behave;
